@@ -1,9 +1,84 @@
-from sqlalchemy.engine.url import make_url
-import sqlalchemy as sa
-from sqlalchemy.exc import ProgrammingError, OperationalError
 import os
 from copy import copy
 
+import sqlalchemy as sa
+from sqlalchemy.engine.url import make_url
+from sqlalchemy.exc import ProgrammingError, OperationalError
+from sqlalchemy_utils.expressions import explain_analyze
+
+
+class PlanAnalysis(object):
+    def __init__(self, plan):
+        self.plan = plan
+
+    @property
+    def node_types(self):
+        types = [self.plan['Node Type']]
+        if 'Plans' in self.plan:
+            for plan in self.plan['Plans']:
+                analysis = PlanAnalysis(plan)
+                types.extend(analysis.node_types)
+        return types
+
+
+class QueryAnalysis(object):
+    def __init__(self, result_set):
+        self.plan = result_set[0]['Plan']
+        self.runtime = result_set[0]['Total Runtime']
+
+    @property
+    def node_types(self):
+        return list(PlanAnalysis(self.plan).node_types)
+
+    def __repr__(self):
+        return '<QueryAnalysis runtime=%r>' % self.runtime
+
+
+def analyze(conn, query):
+    """
+    Analyze query using given connection and return :class:`QueryAnalysis`
+    object. Analysis is performed using database specific EXPLAIN ANALYZE
+    construct and then examining the results into structured format. Currently
+    only PostgreSQL is supported.
+
+
+    Getting query runtime (in database level) ::
+
+
+        from sqlalchemy_utils import analyze
+
+
+        analysis = analyze(conn, 'SELECT * FROM article')
+        analysis.runtime  # runtime as milliseconds
+
+
+    Analyze can be very useful when testing that query doesn't issue a
+    sequential scan (scanning all rows in table). You can for example write
+    simple performance tests this way.::
+
+
+        query = (
+            session.query(Article.name)
+            .order_by(Article.name)
+            .limit(10)
+        )
+        analysis = analyze(self.connection, query)
+        analysis.node_types  # [u'Limit', u'Index Only Scan']
+
+        assert 'Seq Scan' not in analysis.node_types
+
+
+    .. versionadded: 0.26.17
+
+    :param conn: SQLAlchemy Connection object
+    :param query: SQLAlchemy Query object or query as a string
+    """
+    return QueryAnalysis(
+        conn.execute(
+            explain_analyze(query, buffers=True, format='json')
+        ).scalar()
+    )
+
 
 def escape_like(string, escape_char='*'):
     """
